import os
import boto3

from typing import List, Any, Dict
import questionary

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger
from aws_pcluster_bootstrap_helpers.utils.network import get_network_data
from aws_pcluster_bootstrap_helpers.utils.efs import get_efs

logger = setup_logger("configure")
logger.propagate = False


def configure_network(region: str = "us-east-1"):
    network_data = get_network_data(region)
    vpcs = network_data["vpc_list"]
    vpc_subnets = network_data["vpc_subnets"]
    vpc_choices = list(map(lambda x: f"{x['id']} {x['name']}", vpcs))
    vpc_choices_options = list(
        map(lambda x: dict(
            label=f"{x['id']} {x['name']}",
            data=x
        ), vpcs)
    )
    vpc_name = questionary.select("Which VPC?", choices=vpc_choices).ask()
    vpc_choice = list(filter(lambda x: x["label"] == vpc_name, vpc_choices_options))[0]
    vpc_id = vpc_choice["data"]["id"]
    subnets = vpc_subnets[vpc_id]

    subnet_choices = list(map(lambda x: f"{x['id']} {x['name']}", subnets))
    subnet_choices_options = list(
        map(lambda x: dict(label=f"{x['id']} {x['name']}", data=x), subnets)
    )
    head_subnet = questionary.select(
        "Choose a subnet for the head node", choices=subnet_choices
    ).ask()
    compute_subnet = questionary.select(
        "Choose a subnet for the compute node", choices=subnet_choices
    ).ask()
    head_subnet = list(
        filter(lambda x: x["label"] == head_subnet, subnet_choices_options)
    )[0]
    compute_subnet = list(
        filter(lambda x: x["label"] == compute_subnet, subnet_choices_options)
    )[0]
    head_subnet_id = head_subnet["data"]["id"]
    compute_subnet_id = compute_subnet["data"]["id"]

    subnet_ids = [head_subnet_id, compute_subnet_id]
    if head_subnet_id == compute_subnet_id:
        subnet_ids = [head_subnet_id]

    selected_network_data = dict(vpc_id=vpc_id, subnet_ids=subnet_ids)
    return selected_network_data


def prompt_efs(all_efs: List[Any], efs_name="apps") -> List[Any]:
    efs_search = questionary.text(
        f"""Please enter a search pattern for the name of the existing {efs_name} EFS systems.
        None will list all EFS file systems.""",
        default=efs_name,
    ).ask()
    if efs_search:
        filtered_efs = []
        for efs in all_efs:
            if efs_search in efs['Name']:
                filtered_efs.append(efs)
    else:
        filtered_efs = all_efs
    # TODO I don't understand why this doesn't render in the terminal
    # but the other does
    filtered_efs_options = list(
        map(lambda x: dict(
            name=f"[{x['FileSystemId']}] {x['Name']}",
            label=f"[{x['FileSystemId']}] {x['Name']}",
            data=x['FileSystemId']
        ), filtered_efs)
    )
    efs_selected = questionary.select(
        "Choose a efs system", choices=filtered_efs_options
    ).ask()
    efs_choice = list(filter(lambda x: x["label"] == efs_selected, filtered_efs_options))[0]
    return efs_choice['data']


def get_all_acms(regions="us-east-1"):
    client = boto3.client('acm')
    response = client.list_certificates(CertificateStatuses=['ISSUED'])
    all_certs = []
    marker = True
    while marker:
        if 'CertificateSummaryList' in response:
            for cert in response['CertificateSummaryList']:
                all_certs.append(cert)
        if 'NextToken' in response:
            marker = response['NextToken']
            response = client.list_certificates(NextToken=marker)
        else:
            marker = False
    return all_certs


def get_all_hosted_zones(region="us-east-1"):
    client = boto3.client('route53')
    response = client.list_hosted_zones()
    all_zones = []
    marker = True
    while marker:
        if 'HostedZones' in response:
            all_zones.extend(response['HostedZones'])
        if 'NextMarker' in response:
            marker = response['NextMarker']
            response = client.list_hosted_zones(Marker=marker)
        else:
            marker = False
    return all_zones


def get_all_domains(region="us-east-1"):
    # this one only works from us-east-1
    default_region = os.environ['AWS_DEFAULT_REGION']
    os.environ['AWS_DEFAULT_REGION'] = "us-east-1"
    client = boto3.client('route53domains')
    response = client.list_domains()
    all_domains = []
    marker = True
    while marker:
        if 'Domains' in response:
            for domain in response['Domains']:
                all_domains.append(domain['DomainName'])
        if 'NextPageMarker' in response:
            marker = response['NextPageMarker']
            response = client.list_domains(Marker=marker)
        else:
            marker = False
    if default_region and isinstance(default_region, str):
        os.environ['AWS_DEFAULT_REGION'] = default_region
    return all_domains


def configure_dns(region="us-east-1"):
    data = dict(
        acm_domain=None,
        route53_zone_id=None,
        domain=None,
    )
    dns = questionary.confirm("Configure DNS for Jupyterhub?").ask()
    if not dns:
        return data

    # ACMs
    all_acms = get_all_acms()
    acm_options = list(
        map(lambda x: dict(
            name=f"{x['DomainName']}",
            label=f"{x['DomainName']}",
            data=x
        ), all_acms)
    )
    acm_selected = questionary.select(
        "Choose an ACM Certificate", choices=acm_options
    ).ask()
    acm_choice = list(filter(lambda x: x["label"] == acm_selected, acm_options))[0]
    acm_id = acm_choice['data']['DomainName']
    data['acm_domain'] = acm_id

    # Domains
    all_domains = get_all_domains(region=region)
    domain_options = all_domains
    domain_selected = questionary.select(
        "Choose a hosted domain", choices=domain_options
    ).ask()
    domain_choice = domain_selected
    data['domain'] = domain_choice

    # Zones
    all_zones = get_all_hosted_zones(region=region)
    zone_options: List[Dict[str, Any]] = list(
        map(lambda x: dict(
            name=f"{x['Name']}",
            label=f"{x['Name']}",
            data=x
        ), all_zones)
    )
    zone_selected = questionary.select(
        "Choose a hosted zone", choices=zone_options
    ).ask()
    zone_choice = list(filter(lambda x: x["label"] == zone_selected, zone_options))[0]
    zone_id = zone_choice['data']['Id'].split('/')[-1]
    data['route53_zone_id'] = zone_id

    return data


def configure_storage(region="us-east-1"):
    create_apps = questionary.confirm("Create new storage for /apps?", default=False).ask()
    create_scratch = questionary.confirm("Create new storage for /scratch?", default=False).ask()

    data = dict(
        aps_efs_id=None,
        scratch_efs_id=None,
        create_apps=False,
        create_scratch=False,
    )
    all_efs = get_efs(region)
    if not create_apps:
        apps_efs_id = prompt_efs(all_efs, efs_name="apps")
        data['apps_efs_id'] = apps_efs_id
        data['create_apps'] = False
    else:
        data['create_apps'] = True
    if not create_scratch:
        scratch_efs_id = prompt_efs(all_efs, efs_name="scratch")
        data['scratch_efs_id'] = scratch_efs_id
        data['create_scratch'] = False
    else:
        data['create_scratch'] = True

    return data


def configure_public_private_ip(region="us-east-1") -> Dict[str, Any]:
    elastic_ip = questionary.confirm("Use elastic ip for head node? (Usually true)").ask()
    use_public_ip_dns = questionary.confirm("Use the head node public IP for the DNS? (Usually true)").ask()

    return dict(
        head_node_elastic_ip=elastic_ip,
        head_node_public_ip_dns=use_public_ip_dns,
    )


def configure(region: str = "us-east-1"):
    selected_network_data = configure_network(region)
    storage = configure_storage(region)
    elastic_ip = configure_public_private_ip(region=region)
    dns = configure_dns(region)
    data = dict(
        network_data=selected_network_data,
        efs=storage,
        dns=dns,
        elastic_ip=elastic_ip,
    )
    return data



"""Tasks for interacting with shell commands"""

import logging
import os
import sys
import tempfile
from typing import List, Optional, Union

import typing
import os
from pprint import pprint
from datetime import timedelta
from timeit import default_timer as timer

from pathlib import Path
import glob
import subprocess
import shutil
from subprocess import PIPE, STDOUT, Popen
import signal

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger, log_durations

logger = setup_logger("commands")


def run_bash_verbose(command: str, env=None) -> int:
    def pre_exec():
        # Restore default signal disposition and invoke setsid
        for sig in ("SIGPIPE", "SIGXFZ", "SIGXFSZ"):
            if hasattr(signal, sig):
                signal.signal(getattr(signal, sig), signal.SIG_DFL)
        os.setsid()

    output_encoding = "utf-8"
    if not env:
        env = os.environ.copy()

    logger.info(f'Launching subprocess "{command}"')

    sub = Popen(
        ["bash", "-c", command],
        stdout=PIPE,
        stderr=STDOUT,
        cwd=os.getcwd(),
        env=env,
        preexec_fn=pre_exec,
    )

    logger.info("Output:")
    for raw_line in iter(sub.stdout.readline, b""):
        line = raw_line.decode(output_encoding).rstrip()
        logger.info(line)

    sub.wait()
    logger.info("Command exited with return code %s", sub.returncode)
    return sub.returncode



import questionary
import requests
import os
import yaml
from pcluster import utils
from pcluster.api.controllers.cluster_operations_controller import (
    list_clusters,
    describe_cluster,
)
import tempfile

from pcluster.cli.commands.configure.easyconfig import (
    _get_vpcs_and_subnets,
)
from pcluster.utils import load_yaml_dict
from pcluster.schemas.cluster_schema import ClusterSchema
from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

PCLUSTER_VERSION = utils.get_installed_version()

logger = setup_logger("upgrade")


def upgrade(region="us-east-1"):
    """
    Upgrading a cluster keeps :
    1. Network settings
    2. Bootstrap settings
    3. Optional - queue settings
    """
    os.environ["AWS_DEFAULT_REGION"] = region
    clusters = list_clusters(region=region)
    complete_clusters = list(
        filter(lambda x: x.cluster_status == "CREATE_COMPLETE", clusters.clusters)
    )
    if not len(complete_clusters):
        logger.error(
            f"Unable to find any existing clusters. Please create a new cluster."
        )

    cluster_choices = list(map(lambda x: x.cluster_name, complete_clusters))
    cluster_name = questionary.select(
        "Please select a cluster.", choices=cluster_choices
    ).ask()

    cluster_info = describe_cluster(cluster_name, region=region)

    cluster_config = cluster_info.cluster_configuration.url
    cluster_yaml = requests.get(cluster_config).content.decode("utf-8")
    cluster_data = yaml.safe_load(cluster_yaml)
    network_data = _get_vpcs_and_subnets()
    cluster_config = ClusterSchema(cluster_name="clustername").load(cluster_data)
    head_node_subnet = cluster_config.head_node.networking.subnet_id
    compute_subnet_ids = list(
        map(lambda x: x.networking.subnet_ids, cluster_config.scheduling.queues)
    )
    compute_subnet_ids = [item for sublist in compute_subnet_ids for item in sublist]
    list_set = set(compute_subnet_ids)
    compute_subnet_ids = list(list_set)
    subnets = [head_node_subnet]
    for compute_subnet_id in compute_subnet_ids:
        subnets.append(compute_subnet_id)

    list_set = set(subnets)
    subnets = list(list_set)

    vpc_id = None
    for t_vpc_id in network_data["vpc_subnets"].keys():
        subnets = network_data["vpc_subnets"][t_vpc_id]
        for subnet in subnets:
            if subnet["id"] == head_node_subnet:
                vpc_id = t_vpc_id
                break

    # compute_node_subnets = list(
    #     map(cluster_data['Scheduling'])
    # )

    return



from datetime import datetime
from typing import List

from mypy_boto3_logs.client import CloudWatchLogsClient
from mypy_boto3_logs.type_defs import GetLogEventsResponseTypeDef
from mypy_boto3_logs.type_defs import OutputLogEventTypeDef


def get_human_readable_time(last_time_stamp):
    return datetime.fromtimestamp(last_time_stamp / 1000.0).isoformat()


def get_log_events_wrapper(
    client: CloudWatchLogsClient,
    log_group,
    log_stream_name,
    start_time=0,
    skip=0,
    start_from_head=True,
) -> List[OutputLogEventTypeDef]:
    """
    A generator for log items in a single stream. This will yield all the
    items that are available at the current moment.

    Completely stole this from here
    https://airflow.apache.org/docs/apache-airflow/1.10.5/_modules/airflow/contrib/hooks/aws_logs_hook.html

    :param client: boto3.client('cloudwatch')
    :param log_group: The name of the log group.
    :type log_group: str
    :param log_stream_name: The name of the specific stream.
    :type log_stream_name: str
    :param start_time: The time stamp value to start reading the logs from (default: 0).
    :type start_time: int
    :param skip: The number of log entries to skip at the start (default: 0).
        This is for when there are multiple entries at the same timestamp.
    :type skip: int
    :param start_from_head: whether to start from the beginning (True) of the log or
        at the end of the log (False).
    :type start_from_head: bool
    :rtype: dict
    :return: | A CloudWatch log event with the following key-value pairs:
             |   'timestamp' (int): The time in milliseconds of the event.
             |   'message' (str): The log event data.
             |   'ingestionTime' (int): The time in milliseconds the event was ingested.
    """

    next_token = None

    event_count = 1
    while event_count > 0:
        if next_token is not None:
            token_arg = {"nextToken": next_token}
        else:
            token_arg = {}

        response: GetLogEventsResponseTypeDef = client.get_log_events(
            logGroupName=log_group,
            logStreamName=log_stream_name,
            startTime=start_time,
            startFromHead=start_from_head,
            **token_arg,
        )

        events = response["events"]
        event_count = len(events)

        if event_count > skip:
            events = events[skip:]
            skip = 0
        else:
            skip = skip - event_count
            events = []

        for ev in events:
            yield ev

        if "nextForwardToken" in response:
            next_token = response["nextForwardToken"]
        else:
            return


def print_logs(
    client: CloudWatchLogsClient,
    log_stream_name: str,
    log_group: str = "/aws/batch/job",
    start_time: int = 0,
) -> int:
    log_events = get_log_events_wrapper(
        client,
        log_group=log_group,
        log_stream_name=log_stream_name,
        start_time=start_time,
    )

    last_time_stamp = start_time
    for log_event in log_events:
        last_time_stamp = log_event["timestamp"]
        human_timestamp = get_human_readable_time(last_time_stamp)
        message = log_event["message"]
        print(f"[{human_timestamp}] {message}")

    if last_time_stamp > 0:
        last_time_stamp = last_time_stamp + 1

    return last_time_stamp



import pydantic
from typing import Optional, List, Any
from enum import Enum

from enum import Enum, IntEnum
from pydantic import BaseModel, ValidationError
import questionary
from pcluster.cli.commands.configure.easyconfig import (
    _create_vpc_parameters,
    _choose_network_configuration,
    _get_common_supported_az_for_multi_instance_types,
)
from pcluster.cli.commands.configure.easyconfig import (
    _get_vpcs_and_subnets,
)
from pcluster.constants import (
    DEFAULT_MAX_COUNT,
    DEFAULT_MIN_COUNT,
    MAX_NUMBER_OF_COMPUTE_RESOURCES,
    MAX_NUMBER_OF_QUEUES,
    SUPPORTED_SCHEDULERS,
)
from pcluster.utils import error, get_supported_os_for_scheduler
from pcluster.validators.cluster_validators import NameValidator
from pcluster.aws.aws_api import AWSApi
from pcluster.aws.common import get_region
from pcluster.cli.commands.configure.networking import (
    NetworkConfiguration,
    PublicPrivateNetworkConfig,
    automate_subnet_creation,
    automate_vpc_with_subnet_creation,
)
from questionary import Separator

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

logger = setup_logger("configure")


class NetworkConfig(BaseModel):
    vpc_id: str
    subnet_ids: List[str]


class TargetGroupType(str, Enum):
    ip = "ip"
    instance = "instance"


class EFSConfig(BaseModel):
    create_efs: bool = True
    efs_id: Optional[str]
    efs_mount_path: str


class EFSMountConfig(BaseModel):
    efs: List[EFSConfig]


class DNSConfig(BaseModel):
    zone_id: str
    acm_id: str
    target_group_type: Enum
    head_node_elastic_ip: bool = True
    internal_lb: bool = False


class TagsConfig(BaseModel):
    name: str
    namespace: str
    environment: str = "hpc"
    stage: str = "dev"


class PClusterConfigure(BaseModel):
    region: str



import os
import numpy as np
from typing import List, Any, Dict, Optional
import questionary

from pcluster.aws.aws_api import AWSApi
import pandas as pd

from pcluster import utils
from pcluster.constants import MAX_NUMBER_OF_QUEUES, MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER
from datasize import DataSize
from questionary.prompts import common
from questionary.prompts.common import Choice, InquirerControl, Separator
from questionary.question import Question

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

PCLUSTER_VERSION = utils.get_installed_version()
logger = setup_logger("configure-queues")

include_families = [
    "t3a",
    "m5a",
    "m5d",
    "m6i",
    "m6a",
    "c4",
    "c5",
    "c6",
    "r4",
    "r5",
    "r5a",
    "r5ad",
    "r5b",
    "r5d",
    "r5dn",
    "r5n",
    "r6a",
    "r6i",
    "r6id",
    # gpu types
    "g2",
    "g3",
    "g3s",
    "g4ad",
    "g4dn",
    "p2",
    "p3",
    "p3dn",
    "p4dn",
    "p4d",
]
exclude_families = ["m5dn"]
exclude_sizes = ["nano", "metal", "micro", "small", "medium"]
include_mems = [16, 32, 60, 128, 192, 356, 512, 768, 1024, 1536]


def size_in_gib(mib: int) -> int:
    mib_bytes = DataSize(f"{mib}Mi")
    return mib_bytes / mib_bytes.IEC_prefixes["Gi"]


def generate_preferred_families() -> List[Any]:
    return []


def get_instance_types(
    region: str = "us-east-1",
    architecture: str = "x86_64",
    include_families: Optional[List] = include_families,
    exclude_families: Optional[List] = exclude_families,
    include_gpus=True,
    exclude_sizes: Optional[List] = exclude_sizes,
    include_sizes: Optional[List] = None,
    mem_upper_limit: Optional = None,
    mem_lower_limit: Optional = None,
    vcpu_upper_limit: Optional = None,
    vcpu_lower_limit: Optional = None,
    include_mems: Optional[List[Any]] = include_mems,
    n_cpu_queues: int = 6,
    n_gpu_queues: int = 4,
):
    if n_gpu_queues + n_cpu_queues > MAX_NUMBER_OF_QUEUES:
        raise ValueError(
            f"Gpu queues + CPU Queues must be less than {MAX_NUMBER_OF_QUEUES}"
        )
    if include_families is None:
        include_families = []
    if exclude_families is None:
        exclude_families = []
    if exclude_sizes is None:
        exclude_sizes = []
    if include_sizes is None:
        include_sizes = []
    if include_mems is None:
        include_mems = []

    queues = {}
    logger.info(f"Getting instance types for region: {region}")
    os.environ["AWS_DEFAULT_REGION"] = region
    instance_types = AWSApi.instance().ec2.list_instance_types()
    instance_records = []
    instance_type_infos = []
    logger.info(
        f"Getting instance type info for {len(instance_types)} in region {region}"
    )
    for instance_type in instance_types:
        info = AWSApi.instance().ec2.get_instance_type_info(instance_type)
        instance_type = info.__dict__["instance_type_data"]
        family = instance_type["InstanceType"].split(".")[0]
        size = instance_type["InstanceType"].split(".")[1]

        size_in_mib = None
        bytes = None
        gibs = None
        arch = instance_type["ProcessorInfo"]["SupportedArchitectures"][0]
        if "MemoryInfo" in instance_type.keys():
            if "SizeInMib" in instance_type["MemoryInfo"].keys():
                size_in_mib = instance_type["MemoryInfo"]["SizeInMib"]
                bytes = DataSize(f"{size_in_mib}Mi")
                gibs = bytes / bytes.IEC_prefixes["Gi"]
                instance_type["MemoryInfo"]["SizeInGib"] = gibs
            elif "SizeInMiB" in instance_type["MemoryInfo"].keys():
                size_in_mib = instance_type["MemoryInfo"]["SizeInMiB"]
                bytes = DataSize(f"{size_in_mib}Mi")
                gibs = bytes / bytes.IEC_prefixes["Gi"]
                instance_type["MemoryInfo"]["SizeInGib"] = gibs
            else:
                raise ValueError(instance_type["MemoryInfo"])

        gpu = False
        if "GpuInfo" in instance_type:
            if "Gpus" in instance_type["GpuInfo"]:
                if len(instance_type["GpuInfo"]["Gpus"]):
                    gpu = True

        try:
            efa = instance_type["NetworkInfo"]["EfaSupported"]
        except Exception as e:
            efa = False
        instance_record = dict(
            family=family,
            arch=arch,
            size=size,
            efa=efa,
            instance_type=instance_type["InstanceType"],
            vcpus=instance_type["VCpuInfo"]["DefaultVCpus"],
            cores=instance_type["VCpuInfo"]["DefaultCores"],
            size_in_mib=size_in_mib,
            size_in_gibs=gibs,
            gpu=gpu,
        )
        instance_records.append(instance_record)

        instance_type_infos.append(instance_type)
    logger.info(f"Generating data frame")
    df = pd.DataFrame.from_records(instance_records)
    df = df[df["arch"].str.contains(architecture)]
    families = list(df["family"].unique())
    families_not_found = []
    if len(include_families):
        df = df[df["family"].str.contains("|".join(include_families))]
    if len(exclude_families):
        df = df[~df["family"].isin(exclude_families)]
    if len(exclude_sizes):
        df = df[~df["size"].isin(exclude_sizes)]

    # in us-east-1 this gets to about 48 instance types

    if not include_gpus:
        df = df[~df["gpu"]]

    if mem_upper_limit:
        df = df[df["size_in_gibs"] >= mem_upper_limit]
    if mem_lower_limit:
        df = df[df["size_in_gibs"] >= mem_lower_limit]

    if vcpu_upper_limit:
        df = df[df["vcpus"] >= vcpu_upper_limit]
    if vcpu_lower_limit:
        df = df[df["vcpus"] >= vcpu_lower_limit]

    df = df.sort_values(by=["size_in_gibs", "vcpus", "family"])
    cpu_df = df[~df["gpu"]]
    gpu_df = df[df["gpu"]]

    n_gpu_instance_types = n_gpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES
    n_cpu_instance_types = n_cpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES
    if gpu_df.shape[0] <= n_gpu_instance_types:
        # include all the gpus
        gpu_dfs = np.array_split(gpu_df, MAX_NUMBER_OF_COMPUTE_RESOURCES)
        for i, t_gpu_df in enumerate(gpu_dfs):
            n = i + 1
            queues[f"gpu-{n}"] = t_gpu_df.to_dict("records")

    if len(include_mems):
        cpu_df = cpu_df[cpu_df["size_in_gibs"].isin(include_mems)]

    logger.info("Complete get instance types")
    return dict(gpu_df=gpu_df, cpu_df=cpu_df, all_df=df)


def write_instance_types_csvs(gpu_df: pd.DataFrame, cpu_df: pd.DataFrame):
    logger.info("Writing out gpu and cpu instance types...")
    gpu_df.to_csv("pcluster_gpu_instances.csv", index=False)
    cpu_df.to_csv("pcluster_cpu_instances.csv", index=False)


def prompt_for_instance_types(region="us-east-1"):
    dfs = get_instance_types(region=region)
    cpu_df = dfs["cpu_df"]
    gpu_df = dfs["gpu_df"]
    include_gpus = questionary.confirm("Create queues from GPU Instances?").ask()
    if include_gpus:
        cpu_queues = questionary.select(
            "How many cpu queues?", choices=list(range(1, MAX_NUMBER_OF_QUEUES))
        ).ask()
        gpu_queues = MAX_NUMBER_OF_QUEUES - cpu_queues
    else:
        cpu_queues = 10
        gpu_queues = 0

    n_cpu_instance_types = cpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES
    n_gpu_instance_types = gpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES

    cpu_queues_data = []
    cpu_labels = (
        cpu_df["instance_type"]
        + " | "
        + cpu_df["size_in_gibs"].map(str)
        + " gib"
        + " | "
        + cpu_df["vcpus"].map(str)
        + " cores"
    )
    cpu_labels = cpu_labels.map(str)
    cpu_labels = list(cpu_labels)

    gpu_labels = (
        gpu_df["instance_type"]
        + " | "
        + gpu_df["size_in_gibs"].map(str)
        + " gib"
        + " | "
        + gpu_df["vcpus"].map(str)
        + " cores"
    )
    gpu_labels = gpu_labels.map(str)
    gpu_labels = list(gpu_labels)

    cpu_min_mems = cpu_df["size_in_gibs"].min()
    cpu_max_mems = cpu_df["size_in_gibs"].max()
    gpu_min_mems = gpu_df["size_in_gibs"].min()
    gpu_max_mems = gpu_df["size_in_gibs"].max()
    cpu_mems = cpu_df["size_in_gibs"].map(str).unique().tolist()
    gpu_mems = gpu_df["size_in_gibs"].unique().tolist()
    cpu_families = cpu_df["family"].unique().tolist()
    gpu_families = gpu_df["family"].unique().tolist()

    logger.info("###################################################")
    logger.info(f"Begin selection for CPU Queues")
    logger.info(f"Choose a maximum of : {n_cpu_instance_types}")
    logger.info("###################################################")
    cpu_min_mem = questionary.select(
        "CPU min memory", choices=cpu_mems, default=cpu_mems[0]
    ).ask()
    cpu_max_mem = questionary.select(
        "CPU max memory", choices=cpu_mems, default=cpu_mems[-1]
    ).ask()
    t_cpu_df = cpu_df[cpu_df["size_in_gibs"] >= float(cpu_min_mem)]
    t_cpu_df = t_cpu_df[t_cpu_df["size_in_gibs"] <= float(cpu_max_mem)]
    families = t_cpu_df["family"].unique().tolist()
    cpu_families = questionary.checkbox(
        "CPU families",
        choices=list(map(lambda x: Choice(x, checked=True), families)),
    ).ask()
    t_cpu_df = t_cpu_df[t_cpu_df["family"].isin(cpu_families)]

    cpu_labels = (
        t_cpu_df["instance_type"]
        + " | "
        + t_cpu_df["size_in_gibs"].map(str)
        + " gib"
        + " | "
        + t_cpu_df["vcpus"].map(str)
        + " cores"
    )
    cpu_labels = cpu_labels.map(str)
    cpu_labels = list(cpu_labels)

    instance_types = questionary.checkbox(
        f"Select instance types: max {n_cpu_instance_types}",
        choices=cpu_labels,
        validate=lambda x: True
        if len(x) > n_cpu_instance_types
        else f"Please choose a maximum of: {n_cpu_instance_types}",
    ).ask()

    queue_rules = [
        # dev
        dict(
            min_mem=16,
            max_mem=96,
            preferred_families=[
                "m5",
                "m5a",
                "m5ad",
                "m5d",
                "m5dn",
                "m5n",
                "m5zn",
                "m6a",
                "m6i",
                "m6id",
                "m5",
                "m5a",
                "m6a",
                "t3a",
            ],
        ),
        # compute optimized
        dict(
            min_mem=96,
            max_mem=488,
            preferred_families=[
                "c1",
                "c3",
                "c4",
                "c5",
                "c5a",
                "c5ad",
                "c5d",
                "c5n",
                "c6a",
                "c6i",
                "c6id",
                "cc2",
                "c5",
                "c5a",
                "c4",
            ],
        ),
        # mem optimized
        dict(
            min_mem=96,
            max_mem=488,
            preferred_families=[
                "r3",
                "r4",
                "r5",
                "r5a",
                "r5ad",
                "r5b",
                "r5d",
                "r5dn",
                "r5n",
                "r6a",
                "r6i",
                "r6id",
            ],
        ),
        dict(
            min_mem=96,
            max_mem=488,
            prefrred_families=[
                "r3",
                "r4",
                "r5",
                "r5a",
                "r5ad",
                "r5b",
                "r5d",
                "r5dn",
                "r5n",
                "r6a",
                "r6i",
                "r6id",
                "c1",
                "c3",
                "c4",
                "c5",
                "c5a",
                "c5ad",
                "c5d",
                "c5n",
                "c6a",
                "c6i",
                "c6id",
                "cc2",
                "c5",
                "c5a",
                "c4",
            ],
        ),
        dict(min_mem=488, max_mem=768),
        dict(min_mem=488, max_mem=None),
    ]
    queues = {}
    for i, queue_rule in enumerate(queue_rules):
        if queue_rule["min_mem"]:
            cpu_queue = cpu_df[cpu_df["size_in_gibs"] >= 16]
        else:
            cpu_queue = cpu_df
        if queue_rule["max_mem"]:
            cpu_queue = cpu_queue[cpu_queue["size_in_gibs"] <= 96]

        if cpu_queue.shape[0] > MAX_NUMBER_OF_COMPUTE_RESOURCES:
            if queue_rule.get("preferred_families", None):
                cpu_queue = cpu_queue[
                    cpu_queue["family"].isin(queue_rule["preferred_families"])
                ]
            else:
                cpu_queue = cpu_queue[cpu_queue["family"].isin(include_families)]

        queues[f"cpu-{i + 1}"] = cpu_queue
        mems = cpu_queue["size_in_gibs"].unique().tolist()
        min_mem = cpu_queue["size_in_gibs"].min()
        max_mem = cpu_queue["size_in_gibs"].max()
        bins = pd.cut(
            cpu_queue["size_in_gibs"].unique(), bins=MAX_NUMBER_OF_COMPUTE_RESOURCES
        )



import boto3


def get_acms(region: str = "us-east-1"):
    acm_client = boto3.client("acm", region_name=region)

    response = acm_client.list_certificates()

    certificates = response["CertificateSummaryList"]
    while response.get("NextToken"):
        response = acm_client.list_certificates(NextToken=response["NextToken"])
        certificates.extend(response["CertificateSummaryList"])

    return certificates



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages
import versioneer

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

with open("requirements.txt", "r") as fh:
    requirements = [line.strip() for line in fh]

test_requirements = []

setup(
    author="Jillian Rowe",
    author_email="jillian@dabbleofdevops.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="Helpers to bootstrap a AWS PCluster + SLURM + Custom AMIs",
    entry_points={
        "console_scripts": [
            "pcluster-bootstrap-helper=aws_pcluster_bootstrap_helpers.cli:cli",
        ],
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="aws_pcluster_bootstrap_helpers",
    name="aws_pcluster_bootstrap_helpers",
    packages=find_packages(
        include=["aws_pcluster_bootstrap_helpers", "aws_pcluster_bootstrap_helpers.*"]
    ),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/dabble-of-devops-bioanalyze/aws_pcluster_bootstrap_helpers",
    # version="0.5.0",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    zip_safe=False,
)
